@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	27 July 1988
- * Last Edited:	15 March 2004
+ * Last Edited:	6 July 2004
  * 
  * The IMAP toolkit provided in this Distribution is
  * Copyright 1988-2004 University of Washington.
@@ -45,6 +45,7 @@
 #define SMTPSOFTFATAL (long) 421/* SMTP soft fatal code */
 #define SMTPWANTAUTH (long) 505	/* SMTP authentication needed */
 #define SMTPWANTAUTH2 (long) 530/* SMTP authentication needed */
+#define SMTPUNAVAIL (long) 550	/* SMTP mailbox unavailable */
 #define SMTPHARDERROR (long) 554/* SMTP miscellaneous hard failure */
 
 
@@ -303,7 +304,10 @@ long smtp_auth (SENDSTREAM *stream,NETMBX *mb,char *tmp)
 	if (!(at->flags & AU_SECURE)) stream->sensitive = T;
 	if ((*at->client) (smtp_challenge,smtp_response,"smtp",mb,stream,
 			   &trial,usr)) {
-	  if (stream->replycode == SMTPAUTHED) ret = LONGT;
+	  if (stream->replycode == SMTPAUTHED) {
+	    ESMTP.auth = NIL;	/* disable authenticators */
+	    ret = LONGT;
+	  }
 				/* if main program requested cancellation */
 	  else if (!trial) mm_log ("SMTP Authentication cancelled",ERROR);
 	}
@@ -465,6 +469,7 @@ long smtp_mail (SENDSTREAM *stream,char *type,ENVELOPE *env,BODY *body)
     }
 				/* send "MAIL FROM" command */
     switch (smtp_send (stream,type,tmp)) {
+    case SMTPUNAVAIL:		/* mailbox unavailable? */
     case SMTPWANTAUTH:		/* wants authentication? */
     case SMTPWANTAUTH2:
       if (ESMTP.auth) retry = T;/* yes, retry with authentication */
@@ -556,6 +561,7 @@ long smtp_rcpt (SENDSTREAM *stream,ADDRESS *adr,long *error)
 	switch (smtp_send (stream,"RCPT",tmp)) {
 	case SMTPOK:		/* looks good */
 	  break;
+	case SMTPUNAVAIL:	/* mailbox unavailable? */
 	case SMTPWANTAUTH:	/* wants authentication? */
 	case SMTPWANTAUTH2:
 	  if (ESMTP.auth) return T;
