@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	2 February 1994
- * Last Edited:	1 May 2004
+ * Last Edited:	31 August 2004
  *
  * The IMAP tools software provided in this Distribution is
  * Copyright 1988-2004 by the University of Washington
@@ -261,6 +261,7 @@ int main (int argc,char *argv[])
 	      if (last != start) sprintf (t,":%lu,%lu",last,m);
 				/* no, just this message */
 	      else sprintf (t,",%lu",m);
+	      start = last = m;	/* either way, start new range */
 				/* running out of space? */
 	      if ((len - (curlen = (t += strlen (t)) - s)) < 20) {
 		fs_resize ((void **) &s,len += MAILTMPLEN);
@@ -361,10 +362,10 @@ int main (int argc,char *argv[])
     printf ("       %s (copy | move) [-debug] [-verbose] old_mailbox new_mailbox\n",pgm);
     puts   ("        ;; create new mailbox and copy/move messages");
     printf ("       %s (append | appenddelete) [-debug] [-verbose] source destination\n",pgm);
-    puts   ("        ;; prune mailbox of messages matching criteria");
+    puts   ("        ;; copy/move messages to existing mailbox");
     printf ("       %s prune [-debug] [-verbose] mailbox search_criteria\n",
 	    pgm);
-    puts   ("        ;; copy/move messages to existing mailbox");
+    puts   ("        ;; prune mailbox of messages matching criteria");
     printf ("       %s transfer [-debug] [-verbose] [-merge mode] source destination\n",pgm);
     puts   ("        ;; make copy of source hierarchy to destination");
     puts   ("        ;;  -merge modes are prompt, append, or suffix=xxxx");
@@ -432,9 +433,9 @@ int mbxcopy (MAILSTREAM *source,MAILSTREAM *dest,char *dst,int create,int del,
     sprintf (tmp,"1:%lu",ap.msgmax);
     mail_fetchfast (source,tmp);
     if (mail_append_multiple (dest,dst,mm_append,(void *) &ap)) {
-				/* make sure user knows it won */
-      if (verbosep) printf ("[Ok %lu messages(s)]\n",--ap.msgno);
-      if (del && --ap.msgno) {	/* delete source messages */
+      --ap.msgno;		/* make sure user knows it won */
+      if (verbosep) printf ("[Ok %lu messages(s)]\n",ap.msgno);
+      if (del && ap.msgno) {	/* delete source messages */
 	sprintf (tmp,"1:%lu",ap.msgno);
 	mail_flag (source,tmp,"\\Deleted",ST_SET);
 				/* flush moved messages */
@@ -484,9 +485,8 @@ long mm_append (MAILSTREAM *stream,void *data,char **flags,char **date,
     if (elt->answered) strcat (t," \\Answered");
     if (elt->draft) strcat (t," \\Draft");
     if (u = elt->user_flags) do	/* any user flags? */
-      if ((MAILTMPLEN - ((t += strlen (t)) - tmp)) > (long)
-	  (2 + strlen
-	   (t1 = ap->stream->user_flags[find_rightmost_bit (&u)]))) {
+      if ((t1 = ap->stream->user_flags[find_rightmost_bit (&u)]) &&
+	  (MAILTMPLEN - ((t += strlen (t)) - tmp)) > (long) (2 + strlen (t1))){
 	*t++ = ' ';		/* space delimiter */
 	strcpy (t,t1);		/* copy the user flag */
       }
