@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	3 October 1995
- * Last Edited:	4 May 2004
+ * Last Edited:	7 December 2004
  * 
  * The IMAP toolkit provided in this Distribution is
  * Copyright 1988-2004 University of Washington.
@@ -50,29 +50,26 @@ extern int errno;		/* just in case */
 	
 typedef struct mbx_local {
   unsigned int flagcheck: 1;	/* if ping should sweep for flags */
-  unsigned int fullcheck: 1;	/* if ping must sweep flags and expunged */
+  unsigned int expok: 1;	/* if expunging OK in ping */
   unsigned int expunged : 1;	/* if one or more expunged messages */
   int fd;			/* file descriptor for I/O */
+  int ld;			/* lock file descriptor */
   int ffuserflag;		/* first free user flag */
   off_t filesize;		/* file size parsed */
   time_t filetime;		/* last file time */
   time_t lastsnarf;		/* last snarf time */
+  unsigned long lastpid;	/* PID of last writer */
   unsigned char *buf;		/* temporary buffer */
   unsigned long buflen;		/* current size of temporary buffer */
   unsigned long uid;		/* current text uid */
   SIZEDTEXT text;		/* current text */
+  char lock[MAILTMPLEN];	/* buffer to write lock name */
 } MBXLOCAL;
 
 
 /* Convenient access to local data */
 
 #define LOCAL ((MBXLOCAL *) stream->local)
-
-
-/* mbx_update_status() flags */
-
-#define mus_SYNC 1
-#define mus_EXPUNGE 2
 
 /* Function prototypes */
 
@@ -112,6 +109,7 @@ unsigned long mbx_hdrpos (MAILSTREAM *stream,unsigned long msgno,
 			  unsigned long *size,char **hdr);
 unsigned long mbx_rewrite (MAILSTREAM *stream,unsigned long *reclaimed,
 			   long flags);
+long mbx_flaglock (MAILSTREAM *stream);
 
 /* MBX mail routines */
 
@@ -223,11 +221,7 @@ int mbx_isvalid (MAILSTREAM **stream,char *name,char *tmp)
     }
   }
 				/* in case INBOX but not mbx format */
-  else if ((errno == ENOENT) && ((name[0] == 'I') || (name[0] == 'i')) &&
-	   ((name[1] == 'N') || (name[1] == 'n')) &&
-	   ((name[2] == 'B') || (name[2] == 'b')) &&
-	   ((name[3] == 'O') || (name[3] == 'o')) &&
-	   ((name[4] == 'X') || (name[4] == 'x')) && !name[5]) errno = -1;
+  else if ((errno == ENOENT) && !compare_cstring (name,"INBOX")) errno = -1;
   return ret;			/* return what we should */
 }
 
@@ -241,12 +235,14 @@ void *mbx_parameters (long function,void *value)
 {
   void *ret = NIL;
   switch ((int) function) {
+  case GET_INBOXPATH:
+    if (value) ret = mbx_file ((char *) value,"INBOX");
+    break;
   case SET_ONETIMEEXPUNGEATPING:
-    if (value && (((MBXLOCAL *) ((MAILSTREAM *) value)->local)->expunged))
-      ((MBXLOCAL *) ((MAILSTREAM *) value)->local)->fullcheck = T;
+    if (value) ((MBXLOCAL *) ((MAILSTREAM *) value)->local)->expok = T;
   case GET_ONETIMEEXPUNGEATPING:
     if (value) ret = (void *)
-      (((MBXLOCAL *) ((MAILSTREAM *) value)->local)->fullcheck ? VOIDT : NIL);
+      (((MBXLOCAL *) ((MAILSTREAM *) value)->local)->expok ? VOIDT : NIL);
     break;
   }
   return ret;
@@ -314,8 +310,8 @@ long mbx_create (MAILSTREAM *stream,char *mailbox)
       MM_LOG (tmp,ERROR);
       unlink (mbx);		/* delete the file */
     }
-    else {			/* initialize header */
-      memset (tmp,'\0',HDRSIZE);
+    else {
+      memset (tmp,'\0',HDRSIZE);/* initialize header */
       sprintf (s = tmp,"*mbx*\015\012%08lx00000000\015\012",
 	       (unsigned long) time (0));
       for (i = 0; i < NUSERFLAGS; ++i) {
@@ -330,9 +326,10 @@ long mbx_create (MAILSTREAM *stream,char *mailbox)
 	unlink (mbx);		/* delete the file */
       }
       else ret = T;		/* success */
+      close (fd);		/* close file */
     }
-    close (fd);			/* close file, set proper protections */
   }
+				/* set proper protections */
   return ret ? set_mbx_protections (mailbox,mbx) : NIL;
 }
 
@@ -357,7 +354,7 @@ long mbx_delete (MAILSTREAM *stream,char *mailbox)
 
 long mbx_rename (MAILSTREAM *stream,char *old,char *newname)
 {
-  long ret = T;
+  long ret = LONGT;
   char c,*s,tmp[MAILTMPLEN],file[MAILTMPLEN],lock[MAILTMPLEN];
   int fd,ld;
   struct stat sbuf;
@@ -483,8 +480,8 @@ MAILSTREAM *mbx_open (MAILSTREAM *stream)
     mm_log (tmp,ERROR);
   }
   if (stream->rdonly ||
-      (fd = open (tmp,O_BINARY|O_RDWR,NIL)) < 0) {
-    if ((fd = open (tmp,O_BINARY|O_RDONLY,NIL)) < 0) {
+      (fd = open (tmp,O_RDWR|O_BINARY,NIL)) < 0) {
+    if ((fd = open (tmp,O_RDONLY|O_BINARY,NIL)) < 0) {
       sprintf (tmp,"Can't open mailbox: %s",strerror (errno));
       MM_LOG (tmp,ERROR);
       return NIL;
@@ -497,6 +494,7 @@ MAILSTREAM *mbx_open (MAILSTREAM *stream)
 
   stream->local = memset (fs_get (sizeof (MBXLOCAL)),NIL,sizeof (MBXLOCAL));
   LOCAL->fd = fd;		/* bind the file */
+  LOCAL->ld = -1;		/* no flaglock */
   LOCAL->buf = (char *) fs_get (MAXMESSAGESIZE + 1);
   LOCAL->buflen = MAXMESSAGESIZE;
   LOCAL->text.data = (unsigned char *)
@@ -517,7 +515,7 @@ MAILSTREAM *mbx_open (MAILSTREAM *stream)
   LOCAL->filesize = HDRSIZE;	/* initialize parsed file size */
 				/* time not set up yet */
   LOCAL->lastsnarf = LOCAL->filetime = 0;
-  LOCAL->fullcheck = LOCAL->flagcheck = NIL;
+  LOCAL->expok = LOCAL->flagcheck = NIL;
   stream->sequence++;		/* bump sequence number */
 				/* parse mailbox */
   stream->nmsgs = stream->recent = 0;
@@ -550,7 +548,7 @@ void mbx_close (MAILSTREAM *stream,long options)
 				/* do an expunge if requested */
     if (options & CL_EXPUNGE) mbx_expunge (stream);
     else {			/* otherwise do a checkpoint to purge */
-      LOCAL->fullcheck = T;	/*  possible expunged messages */
+      LOCAL->expok = T;		/*  possible expunged messages */
       mbx_ping (stream);
     }
     stream->silent = silent;	/* restore previous status */
@@ -639,18 +637,22 @@ char *mbx_header (MAILSTREAM *stream,unsigned long msgno,unsigned long *length,
 long mbx_text (MAILSTREAM *stream,unsigned long msgno,STRING *bs,long flags)
 {
   unsigned long i,j;
+  char *s = LOCAL->text.data;
   MESSAGECACHE *elt;
 				/* UID call "impossible" */
   if (flags & FT_UID) return NIL;
 				/* get message status */
   elt = mbx_elt (stream,msgno,NIL);
 				/* if message not seen */
-  if (!(flags & FT_PEEK) && !elt->seen) {
+  if (!(flags & FT_PEEK) && !elt->seen && mbx_flaglock (stream)) {
     elt->seen = T;		/* mark message as seen */
 				/* recalculate status */
-    mbx_update_status (stream,msgno,mus_SYNC);
+    mbx_update_status (stream,msgno,NIL);
     MM_FLAGS (stream,msgno);
+				/* update flags */
+    mbx_flag (stream,NIL,NIL,NIL);
   }
+  if (!LOCAL) i = 0;		/* mbx_flaglock() could have aborted */
 				/* in case previous text cached */
   if (elt->private.uid == LOCAL->uid)
     i = elt->rfc822_size - elt->private.msg.header.text.size;
@@ -666,11 +668,10 @@ long mbx_text (MAILSTREAM *stream,unsigned long msgno,STRING *bs,long flags)
       LOCAL->text.data = (unsigned char *) fs_get ((LOCAL->text.size = i) + 1);
     }
 				/* slurp the data */
-    read (LOCAL->fd,LOCAL->text.data,i);
+    read (LOCAL->fd,s = LOCAL->text.data,i);
     LOCAL->text.data[i] = '\0';	/* tie off string */
   }
-				/* set up stringstruct */
-  INIT (bs,mail_string,LOCAL->text.data,i);
+  INIT (bs,mail_string,s,i);	/* set up stringstruct */
   return T;			/* success */
 }
 
@@ -685,15 +686,22 @@ void mbx_flag (MAILSTREAM *stream,char *sequence,char *flag,long flags)
 {
   time_t tp[2];
   struct stat sbuf;
-  if (!stream->rdonly) {	/* make sure the update takes */
+  unsigned long oldpid = LOCAL->lastpid;
+				/* make sure the update takes */
+  if (!stream->rdonly && LOCAL && (LOCAL->fd >= 0) && (LOCAL->ld >= 0)) {
     fsync (LOCAL->fd);
     fstat (LOCAL->fd,&sbuf);	/* get current write time */
     tp[1] = LOCAL->filetime = sbuf.st_mtime;
-				/* update header */
-    if ((LOCAL->ffuserflag < NUSERFLAGS) &&
-	stream->user_flags[LOCAL->ffuserflag]) mbx_update_header (stream);
+				/* we are the last flag updater */
+    LOCAL->lastpid = (unsigned long) getpid ();
+				/* update header if needed */
+    if (((LOCAL->ffuserflag < NUSERFLAGS) &&
+	 stream->user_flags[LOCAL->ffuserflag]) || (oldpid != LOCAL->lastpid))
+      mbx_update_header (stream);
     tp[0] = time (0);		/* make sure read comes after all that */
     utime (stream->mailbox,tp);
+    unlockfd (LOCAL->ld,LOCAL->lock);
+    LOCAL->ld = -1;
   }
 }
 
@@ -705,19 +713,7 @@ void mbx_flag (MAILSTREAM *stream,char *sequence,char *flag,long flags)
 
 void mbx_flagmsg (MAILSTREAM *stream,MESSAGECACHE *elt)
 {
-  unsigned long i;
-  struct stat sbuf;
-				/* maybe need to do a flagcheck? */
-  if (LOCAL->filetime && !LOCAL->flagcheck) {
-    fstat (LOCAL->fd,&sbuf);	/* get current write time */
-    if (LOCAL->filetime < sbuf.st_mtime) {
-      LOCAL->flagcheck = T;	/* invalidate cache and mark flagcheck */
-      for (i = 1; i <= stream->nmsgs; ++i) mail_elt (stream,i)->valid = NIL;
-    }
-    LOCAL->filetime = 0;	/* don't do this test for any other messages */
-  }
-				/* recalculate status */
-  mbx_update_status (stream,elt->msgno,NIL);
+  if (mbx_flaglock (stream)) mbx_update_status (stream,elt->msgno,NIL);
 }
 
 /* MBX mail ping mailbox
@@ -727,50 +723,61 @@ void mbx_flagmsg (MAILSTREAM *stream,MESSAGECACHE *elt)
 
 long mbx_ping (MAILSTREAM *stream)
 {
-  unsigned long i;
-  long r = T;
+  unsigned long i,pos;
+  long ret = NIL;
   int ld;
   char lock[MAILTMPLEN];
+  MESSAGECACHE *elt;
   struct stat sbuf;
   if (stream && LOCAL) {	/* only if stream already open */
     int snarf = stream->inbox && !stream->rdonly;
+    ret = LONGT;		/* assume OK */
     fstat (LOCAL->fd,&sbuf);	/* get current file poop */
-    if (!LOCAL->fullcheck) {	/* don't bother if already full checking */
-				/* if pending expunged messages */
-      if (LOCAL->expunged && mail_parameters (NIL,GET_EXPUNGEATPING,NIL))
-	LOCAL->fullcheck = T;	/* upgrade to expunged message checking */
-				/* else if external modification */
-      else if (LOCAL->filetime && (LOCAL->filetime < sbuf.st_mtime))
-	LOCAL->flagcheck = T;	/* upgrade to flag checking */
-    }
-    if (LOCAL->flagcheck) {	/* sweep mailbox for changed message status */
-      for (i = 1; i <= stream->nmsgs; )
-	if (mbx_elt (stream,i,LOCAL->fullcheck)) ++i;
-      LOCAL->flagcheck = NIL;	/* got all the updates */
-    }
-    if (((i = (sbuf.st_size - LOCAL->filesize) || !stream->nmsgs) || snarf) &&
+				/* allow expunge if permitted at ping */
+    if (mail_parameters (NIL,GET_EXPUNGEATPING,NIL)) LOCAL->expok = T;
+				/* if external modification */
+    if (LOCAL->filetime && (LOCAL->filetime < sbuf.st_mtime))
+      LOCAL->flagcheck = T;	/* upgrade to flag checking */
+				/* new mail or flagcheck handling needed? */
+    if (((i = (sbuf.st_size - LOCAL->filesize)) || LOCAL->flagcheck ||
+	 !stream->nmsgs || snarf) &&
 	((ld = lockfd (LOCAL->fd,lock,LOCK_EX)) >= 0)) {
-				/* parse new messages in mailbox */
-      if (i) r = mbx_parse (stream);
-      if (LOCAL && snarf) {	/* snarf new messages if still OK */
+      if (LOCAL->flagcheck) {	/* sweep mailbox for changed message status */
+	if (ret = mbx_parse (stream)) {
+	  LOCAL->filetime = sbuf.st_mtime;
+	  for (i = 1; i <= stream->nmsgs; )
+	    if (mbx_elt (stream,i,LOCAL->expok)) ++i;
+	  LOCAL->flagcheck =NIL;/* got all the updates */
+	}
+      }
+      else if (i) ret = mbx_parse (stream);
+      if (ret && snarf) {	/* snarf new messages if still OK */
 	mbx_snarf (stream);
-	r = mbx_parse (stream);	/* parse snarfed messages */
+				/* parse snarfed messages */
+	ret = mbx_parse (stream);
       }
       unlockfd (ld,lock);	/* release shared parse/append permission */
     }
-    if (r && LOCAL->fullcheck) {/* full check requested? */
-				/* no more full check or pending expunge */
-      LOCAL->fullcheck = LOCAL->expunged = NIL;
-      if (!stream->rdonly) {	/* rewrite if not readonly */
+    if (ret) {			/* must still be alive */
+      if (!LOCAL->expunged)	/* look for holes if none known yet */
+	for (i = 1, pos = HDRSIZE;
+	     !LOCAL->expunged && (i <= stream->nmsgs);
+	     i++, pos += elt->private.special.text.size + elt->rfc822_size)
+	  if ((elt = mail_elt (stream,i))->private.special.offset != pos)
+	    LOCAL->expunged = T;/* found a hole */
+				/* burp any holes */
+      if (LOCAL->expunged && !stream->rdonly) {
 	if (mbx_rewrite (stream,&i,NIL)) fatal ("expunge on check");
-	if (i) {
+	if (i) {		/* any space reclaimed? */
+	  LOCAL->expunged = NIL;/* no more pending expunge */
 	  sprintf (LOCAL->buf,"Reclaimed %lu bytes of expunged space",i);
 	  MM_LOG (LOCAL->buf,(long) NIL);
 	}
       }
+      LOCAL->expok = NIL;	/* no more expok */
     }
   }
-  return r;			/* return result of the parse */
+  return ret;			/* return result of the parse */
 }
 
 /* MBX mail check mailbox (reparses status too)
@@ -779,8 +786,7 @@ long mbx_ping (MAILSTREAM *stream)
 
 void mbx_check (MAILSTREAM *stream)
 {
-				/* mark that a check is desired */
-  if (LOCAL) LOCAL->fullcheck = LOCAL->flagcheck = T;
+  if (LOCAL) LOCAL->expok = T;	/* mark that a check is desired */
   if (mbx_ping (stream)) MM_LOG ("Check completed",(long) NIL);
 }
 
@@ -791,30 +797,20 @@ void mbx_check (MAILSTREAM *stream)
 
 void mbx_expunge (MAILSTREAM *stream)
 {
-  struct stat sbuf;
-  unsigned long i,nexp,reclaimed;
+  unsigned long nexp,reclaimed;
   if (!mbx_ping (stream));	/* do nothing if stream dead */
   else if (stream->rdonly)	/* won't do on readonly files! */
     MM_LOG ("Expunge ignored on readonly mailbox",WARN);
-  else {			/* see if will need a flagcheck */
-    if (LOCAL->filetime && !LOCAL->flagcheck) {
-      fstat (LOCAL->fd,&sbuf);	/* get current write time */
-				/* if externally altered, must flagcheck */
-      if (LOCAL->filetime < sbuf.st_mtime) LOCAL->flagcheck = T;
-    }
-    if (LOCAL->flagcheck)	/* sweep flags if need flagcheck */
-      for (i = 1; i <= stream->nmsgs; ++i) mbx_elt (stream,i,T)->valid = T;
 				/* if expunged any messages */
-    if (nexp = mbx_rewrite (stream,&reclaimed,T)) {
-      sprintf (LOCAL->buf,"Expunged %lu messages",nexp);
-      MM_LOG (LOCAL->buf,(long) NIL);
-    }
-    else if (reclaimed) {	/* or if any prior expunged space reclaimed */
-      sprintf (LOCAL->buf,"Reclaimed %lu bytes of expunged space",reclaimed);
-      MM_LOG (LOCAL->buf,(long) NIL);
-    }
-    else MM_LOG ("No messages deleted, so no update needed",(long) NIL);
+  else if (nexp = mbx_rewrite (stream,&reclaimed,T)) {
+    sprintf (LOCAL->buf,"Expunged %lu messages",nexp);
+    MM_LOG (LOCAL->buf,(long) NIL);
   }
+  else if (reclaimed) {		/* or if any prior expunged space reclaimed */
+    sprintf (LOCAL->buf,"Reclaimed %lu bytes of expunged space",reclaimed);
+    MM_LOG (LOCAL->buf,(long) NIL);
+  }
+  else MM_LOG ("No messages deleted, so no update needed",(long) NIL);
 }
 
 /* MBX mail snarf messages from system inbox
@@ -830,14 +826,13 @@ void mbx_snarf (MAILSTREAM *stream)
   MESSAGECACHE *elt;
   MAILSTREAM *sysibx = NIL;
 				/* give up if can't get exclusive permission */
-  if ((time (0) < (LOCAL->lastsnarf + 30)) ||
-      !strcmp (sysinbox (),stream->mailbox)) return;
-  MM_CRITICAL (stream);		/* go critical */
-				/* see if anything there */
-  if (!stat (sysinbox (),&sbuf) && sbuf.st_size) {
-    fstat (LOCAL->fd,&sbuf);	/* yes, get current file size */
-				/* sizes match and can get sysibx mailbox? */
-    if ((sbuf.st_size == LOCAL->filesize) && 
+  if ((time (0) >= (LOCAL->lastsnarf +
+		    (long) mail_parameters (NIL,GET_SNARFINTERVAL,NIL))) &&
+      strcmp (sysinbox (),stream->mailbox)) {
+    MM_CRITICAL (stream);	/* go critical */
+				/* sizes match and anything in sysinbox? */
+    if (!stat (sysinbox (),&sbuf) && sbuf.st_size &&
+	!fstat (LOCAL->fd,&sbuf) && (sbuf.st_size == LOCAL->filesize) && 
 	(sysibx = mail_open (sysibx,sysinbox (),OP_SILENT)) &&
 	(!sysibx->rdonly) && (r = sysibx->nmsgs)) {
 				/* yes, go to end of file in our mailbox */
@@ -847,8 +842,9 @@ void mbx_snarf (MAILSTREAM *stream)
 				/* snarf message from system INBOX */
 	hdr = cpystr (mail_fetchheader_full (sysibx,i,NIL,&hdrlen,NIL));
 	txt = mail_fetchtext_full (sysibx,i,&txtlen,FT_PEEK);
+				/* if have a message */
 	if (j = hdrlen + txtlen) {
-				/* calculate header line */
+				/* build header line */
 	  mail_date (LOCAL->buf,elt = mail_elt (sysibx,i));
 	  sprintf (LOCAL->buf + strlen (LOCAL->buf),
 		   ",%lu;00000000%04x-00000000\015\012",j,(unsigned)
@@ -880,9 +876,9 @@ void mbx_snarf (MAILSTREAM *stream)
       LOCAL->filetime = sbuf.st_mtime;
     }
     if (sysibx) mail_close (sysibx);
+    MM_NOCRITICAL (stream);	/* release critical */
+    LOCAL->lastsnarf = time (0);/* note time of last snarf */
   }
-  MM_NOCRITICAL (stream);	/* release critical */
-  LOCAL->lastsnarf = time (0);	/* note time of last snarf */
 }
 
 /* MBX mail copy message(s)
@@ -978,20 +974,15 @@ long mbx_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
   unlockfd (ld,lock);		/* release exclusive parse/append permission */
   MM_NOCRITICAL (stream);	/* release critical */
 				/* delete all requested messages */
-  if (ret && (options & CP_MOVE)) {
+  if (ret && (options & CP_MOVE) && mbx_flaglock (stream)) {
     for (i = 1; i <= stream->nmsgs; i++) if (mail_elt (stream,i)->sequence) {
 				/* mark message deleted */
       mbx_elt (stream,i,NIL)->deleted = T;
 				/* recalculate status */
       mbx_update_status (stream,i,NIL);
     }
-    if (!stream->rdonly) {	/* make sure the update takes */
-      fsync (LOCAL->fd);
-      fstat (LOCAL->fd,&sbuf);	/* get current write time */
-      tp[1] = LOCAL->filetime = sbuf.st_mtime;
-      tp[0] = time (0);		/* make sure atime remains greater */
-      utime (stream->mailbox,tp);
-    }
+				/* update flags */
+    mbx_flag (stream,NIL,NIL,NIL);
   }
   return ret;
 }
@@ -1020,13 +1011,8 @@ long mbx_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
 				/* make sure valid mailbox */
   if (!mbx_isvalid (&dstream,mailbox,tmp)) switch (errno) {
   case ENOENT:			/* no such file? */
-    if (((mailbox[0] == 'I') || (mailbox[0] == 'i')) &&
-	((mailbox[1] == 'N') || (mailbox[1] == 'n')) &&
-	((mailbox[2] == 'B') || (mailbox[2] == 'b')) &&
-	((mailbox[3] == 'O') || (mailbox[3] == 'o')) &&
-	((mailbox[4] == 'X') || (mailbox[4] == 'x')) && !mailbox[5]) {
+    if (!compare_cstring (mailbox,"INBOX"))
       mbx_create (dstream = stream ? stream : user_flags (&mbxproto),"INBOX");
-    }
     else {
       MM_NOTIFY (stream,"[TRYCREATE] Must create mailbox before append",NIL);
       return NIL;
@@ -1179,6 +1165,16 @@ long mbx_parse (MAILSTREAM *stream)
   }
   LOCAL->ffuserflag = (int) i;	/* first free user flag */
 
+				/* get current last flag updater PID */
+  i = (isxdigit (LOCAL->buf[HDRSIZE-10]) && isxdigit (LOCAL->buf[HDRSIZE-9]) &&
+       isxdigit (LOCAL->buf[HDRSIZE-8]) && isxdigit (LOCAL->buf[HDRSIZE-7]) &&
+       isxdigit (LOCAL->buf[HDRSIZE-6]) && isxdigit (LOCAL->buf[HDRSIZE-5]) &&
+       isxdigit (LOCAL->buf[HDRSIZE-4]) && isxdigit (LOCAL->buf[HDRSIZE-3]) &&
+       (LOCAL->buf[HDRSIZE-2] == '\015') && (LOCAL->buf[HDRSIZE-1] == '\012'))?
+    strtoul (LOCAL->buf + HDRSIZE - 8,NIL,16) : 0;
+				/* set flagcheck if lastpid changed */
+  if (LOCAL->lastpid && (LOCAL->lastpid != i)) LOCAL->flagcheck = T;
+  LOCAL->lastpid = i;		/* set as last PID */
   stream->silent = T;		/* don't pass up mm_exists() events yet */
   while (sbuf.st_size - curpos){/* while there is stuff to parse */
 				/* get to that position in the file */
@@ -1438,6 +1434,7 @@ void mbx_update_header (MAILSTREAM *stream)
   stream->kwd_create = (i < NUSERFLAGS) ? T : NIL;
 				/* write reserved lines */
   while (i++ < NUSERFLAGS) strcat (s,"\015\012");
+  sprintf (LOCAL->buf + HDRSIZE - 10,"%08lx\015\012",LOCAL->lastpid);
   while (T) {			/* rewind file */
     lseek (LOCAL->fd,CYGKLUDGEOFFSET,L_SET);
 				/* write new header */
@@ -1456,7 +1453,6 @@ void mbx_update_header (MAILSTREAM *stream)
 
 void mbx_update_status (MAILSTREAM *stream,unsigned long msgno,long flags)
 {
-  time_t tp[2];
   struct stat sbuf;
   MESSAGECACHE *elt = mail_elt (stream,msgno);
 				/* readonly */
@@ -1484,10 +1480,9 @@ void mbx_update_status (MAILSTREAM *stream,unsigned long msgno,long flags)
 	       elt->private.special.text.size,(char *) LOCAL->buf);
       fatal (LOCAL->buf+50);
     }
-    LOCAL->buf[13] = '\0';	/* tie off buffer */
 				/* print new flag string */
     sprintf (LOCAL->buf,"%08lx%04x-%08lx",elt->user_flags,(unsigned)
-	     (((elt->deleted && (flags & mus_EXPUNGE)) ?
+	     (((elt->deleted && flags) ?
 	       fEXPUNGED : (strtoul (LOCAL->buf+9,NIL,16)) & fEXPUNGED) +
 	      (fSEEN * elt->seen) + (fDELETED * elt->deleted) +
 	      (fFLAGGED * elt->flagged) + (fANSWERED * elt->answered) +
@@ -1500,13 +1495,6 @@ void mbx_update_status (MAILSTREAM *stream,unsigned long msgno,long flags)
       MM_NOTIFY (stream,strerror (errno),WARN);
       MM_DISKERROR (stream,errno,T);
     }
-    if (flags & mus_SYNC) {	/* sync if requested */
-      fsync (LOCAL->fd);
-      fstat (LOCAL->fd,&sbuf);	/* get new write time */
-      tp[1] = LOCAL->filetime = sbuf.st_mtime;
-      tp[0] = time (0);		/* make sure read is later */
-      utime (stream->mailbox,tp);
-    }
   }
 }
 
@@ -1534,25 +1522,8 @@ unsigned long mbx_hdrpos (MAILSTREAM *stream,unsigned long msgno,
 				/* is header size known? */ 
   if (*size = elt->private.msg.header.text.size) return ret;
 				/* paranoia check */
-  if (LOCAL->buflen < (HDRBUFLEN + SLOP)) {
-    fs_give ((void **) &LOCAL->buf);
-#if 0
-    /* This causes GCC 2.95 to spew the following crud:
-     *	mbx.c: In function `mbx_hdrpos':
-     *	mbx.c:1371: internal error--unrecognizable insn:
-     *	(insn 78 76 80 (set (reg:SI 129)
-     *		(minus:SI (const_int 4 [0x4])
-     *		    (const_int -4096 [0xfffff000]))) -1 (nil)
-     *	    (expr_list:REG_EQUAL (const_int 4100 [0x1004])
-     *		(nil)))
-     */
-    LOCAL->buf = (char *) fs_get ((LOCAL->buflen = HDRBUFLEN) + SLOP);
-#else
-    LOCAL->buflen = HDRBUFLEN;	/* help the crappy compiler work */
-    LOCAL->buf = (char *) fs_get (LOCAL->buflen + SLOP);
-#endif
-  }
-
+  if (LOCAL->buflen < (HDRBUFLEN + SLOP))
+    fatal ("LOCAL->buf smaller than HDRBUFLEN");
   lseek (LOCAL->fd,ret,L_SET);	/* get to header position */
 				/* read HDRBUFLEN chunks with 4 byte slop */
   for (done = siz = 0, s = LOCAL->buf;
@@ -1622,8 +1593,18 @@ unsigned long mbx_rewrite (MAILSTREAM *stream,unsigned long *reclaimed,
     MM_LOG ("Unable to lock mailbox for rewrite",ERROR);
     return *reclaimed = 0;
   }
-				/* make sure see any newly-arrived messages */
-  if (!mbx_parse (stream)) return NIL;
+  fstat (LOCAL->fd,&sbuf);	/* get current write time */
+  if (LOCAL->filetime && !LOCAL->flagcheck &&
+      (LOCAL->filetime < sbuf.st_mtime)) LOCAL->flagcheck = T;
+  if (!mbx_parse (stream)) {	/* make sure see any newly-arrived messages */
+    unlockfd (ld,lock);		/* failed?? */
+    return *reclaimed = 0;
+  }
+  if (LOCAL->flagcheck) {	/* sweep flags if need flagcheck */
+    LOCAL->filetime = sbuf.st_mtime;
+    for (i = 1; i <= stream->nmsgs; ++i) mbx_elt (stream,i,NIL);
+    LOCAL->flagcheck = NIL;
+  }
 
 				/* get exclusive access */
   if (!flock (LOCAL->fd,LOCK_EX|LOCK_NB)) {
@@ -1696,7 +1677,7 @@ unsigned long mbx_rewrite (MAILSTREAM *stream,unsigned long *reclaimed,
     else for (i = 1,n = *reclaimed = 0; i <= stream->nmsgs; ) {
       if (elt = mbx_elt (stream,i,T)) {
 	if (elt->deleted) {	/* make deleted message invisible */
-	  mbx_update_status (stream,elt->msgno,mus_EXPUNGE);
+	  mbx_update_status (stream,elt->msgno,LONGT);
 				/* notify upper levels */
 	  mail_expunged (stream,i);
 	  n++;			/* count up one more expunged message */
@@ -1719,3 +1700,37 @@ unsigned long mbx_rewrite (MAILSTREAM *stream,unsigned long *reclaimed,
   mail_recent (stream,recent);
   return n;			/* return number of expunged messages */
 }
+
+/* MBX mail lock for flag updating
+ * Accepts: stream
+ * Returns: T if successful, NIL if failure
+ */
+
+long mbx_flaglock (MAILSTREAM *stream)
+{
+  struct stat sbuf;
+  unsigned long i;
+  int ld;
+  char lock[MAILTMPLEN];
+				/* no-op if readonly or already locked */
+  if (!stream->rdonly && LOCAL && (LOCAL->fd >= 0) && (LOCAL->ld < 0)) {
+				/* lock now */
+    if ((ld = lockfd (LOCAL->fd,lock,LOCK_EX)) < 0) return NIL;
+    if (!LOCAL->flagcheck) {	/* don't do this if flagcheck already needed */
+      if (LOCAL->filetime) {	/* know previous time? */
+	fstat (LOCAL->fd,&sbuf);/* get current write time */
+	if (LOCAL->filetime < sbuf.st_mtime) LOCAL->flagcheck = T;
+	LOCAL->filetime = 0;	/* don't do this test for any other messages */
+      }
+      if (!mbx_parse (stream)) {/* parse mailbox */
+	unlockfd (ld,lock);	/* shouldn't happen */
+	return NIL;
+      }
+      if (LOCAL->flagcheck)	/* invalidate cache if flagcheck */
+	for (i = 1; i <= stream->nmsgs; ++i) mail_elt (stream,i)->valid = NIL;
+    }
+    LOCAL->ld = ld;		/* copy to stream for subseuent calls */
+    memcpy (LOCAL->lock,lock,MAILTMPLEN);
+  }
+  return LONGT;
+}
