@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 May 1990
- * Last Edited:	4 May 2004
+ * Last Edited:	7 December 2004
  * 
  * The IMAP toolkit provided in this Distribution is
  * Copyright 1988-2004 University of Washington.
@@ -194,11 +194,7 @@ int mtx_isvalid (char *name,char *tmp)
     }
   }
 				/* in case INBOX but not mtx format */
-  else if ((errno == ENOENT) && ((name[0] == 'I') || (name[0] == 'i')) &&
-	   ((name[1] == 'N') || (name[1] == 'n')) &&
-	   ((name[2] == 'B') || (name[2] == 'b')) &&
-	   ((name[3] == 'O') || (name[3] == 'o')) &&
-	   ((name[4] == 'X') || (name[4] == 'x')) && !name[5]) errno = -1;
+  else if ((errno == ENOENT) && !compare_cstring (name,"INBOX")) errno = -1;
   return ret;			/* return what we should */
 }
 
@@ -210,7 +206,13 @@ int mtx_isvalid (char *name,char *tmp)
 
 void *mtx_parameters (long function,void *value)
 {
-  return NIL;
+  void *ret = NIL;
+  switch ((int) function) {
+  case GET_INBOXPATH:
+    if (value) ret = mtx_file ((char *) value,"INBOX");
+    break;
+  }
+  return ret;
 }
 
 
@@ -611,6 +613,7 @@ long mtx_ping (MAILSTREAM *stream)
 	(LOCAL->filetime < sbuf.st_mtime)) LOCAL->shouldcheck = T;
 				/* check for changed message status */
     if (LOCAL->mustcheck || LOCAL->shouldcheck) {
+      LOCAL->filetime = sbuf.st_mtime;
       if (LOCAL->shouldcheck)	/* babble when we do this unilaterally */
 	MM_NOTIFY (stream,"[CHECK] Checking for flag updates",NIL);
       while (i <= stream->nmsgs) mtx_elt (stream,i++);
@@ -666,15 +669,14 @@ void mtx_snarf (MAILSTREAM *stream)
   MAILSTREAM *sysibx = NIL;
   int ld;
 				/* give up if can't get exclusive permission */
-  if ((time (0) < (LOCAL->lastsnarf + 30)) ||
-      (!strcmp (sysinbox (),stream->mailbox)) ||
-      ((ld = lockfd (LOCAL->fd,lock,LOCK_EX)) < 0)) return;
-  MM_CRITICAL (stream);		/* go critical */
-				/* see if anything there */
-  if (!stat (sysinbox (),&sbuf) && sbuf.st_size) {
-    fstat (LOCAL->fd,&sbuf);	/* yes, get current file size */
-				/* sizes match and can get sysibx mailbox? */
-    if ((sbuf.st_size == LOCAL->filesize) && 
+  if ((time (0) >= (LOCAL->lastsnarf +
+		    (long) mail_parameters (NIL,GET_SNARFINTERVAL,NIL))) &&
+      strcmp (sysinbox (),stream->mailbox) &&
+      ((ld = lockfd (LOCAL->fd,lock,LOCK_EX)) >= 0)) {
+    MM_CRITICAL (stream);	/* go critical */
+				/* sizes match and anything in sysinbox? */
+    if (!stat (sysinbox (),&sbuf) && sbuf.st_size &&
+	!fstat (LOCAL->fd,&sbuf) && (sbuf.st_size == LOCAL->filesize) && 
 	(sysibx = mail_open (sysibx,sysinbox (),OP_SILENT)) &&
 	(!sysibx->rdonly) && (r = sysibx->nmsgs)) {
 				/* yes, go to end of file in our mailbox */
@@ -684,6 +686,7 @@ void mtx_snarf (MAILSTREAM *stream)
 				/* snarf message from system INBOX */
 	hdr = cpystr (mail_fetchheader_full (sysibx,i,NIL,&hdrlen,NIL));
 	txt = mail_fetchtext_full (sysibx,i,&txtlen,FT_PEEK);
+				/* if have a message */
 	if (j = hdrlen + txtlen) {
 				/* calculate header line */
 	  mail_date (LOCAL->buf,elt = mail_elt (sysibx,i));
@@ -699,6 +702,7 @@ void mtx_snarf (MAILSTREAM *stream)
 	}
 	fs_give ((void **) &hdr);
       }
+
 				/* make sure all the updates take */
       if (fsync (LOCAL->fd)) r = 0;
       if (r) {			/* delete all the messages we copied */
@@ -716,10 +720,10 @@ void mtx_snarf (MAILSTREAM *stream)
       LOCAL->filetime = sbuf.st_mtime;
     }
     if (sysibx) mail_close (sysibx);
+    MM_NOCRITICAL (stream);	/* release critical */
+    unlockfd (ld,lock);		/* release exclusive parse/append permission */
+    LOCAL->lastsnarf = time (0);/* note time of last snarf */
   }
-  MM_NOCRITICAL (stream);	/* release critical */
-  unlockfd (ld,lock);		/* release exclusive parse/append permission */
-  LOCAL->lastsnarf = time (0);	/* note time of last snarf */
 }
 
 /* MTX mail expunge mailbox
@@ -964,12 +968,7 @@ long mtx_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
 				/* make sure valid mailbox */
   if (!mtx_isvalid (mailbox,tmp)) switch (errno) {
   case ENOENT:			/* no such file? */
-    if (((mailbox[0] == 'I') || (mailbox[0] == 'i')) &&
-	((mailbox[1] == 'N') || (mailbox[1] == 'n')) &&
-	((mailbox[2] == 'B') || (mailbox[2] == 'b')) &&
-	((mailbox[3] == 'O') || (mailbox[3] == 'o')) &&
-	((mailbox[4] == 'X') || (mailbox[4] == 'x')) && !mailbox[5])
-      dummy_create (NIL,"INBOX.MTX");
+    if (!compare_cstring (mailbox,"INBOX")) dummy_create (NIL,"INBOX.MTX");
     else {
       MM_NOTIFY (stream,"[TRYCREATE] Must create mailbox before append",NIL);
       return NIL;
