@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	11 April 1989
- * Last Edited: 20 February 2004
+ * Last Edited: 2 September 2004
  * 
  * The IMAP toolkit provided in this Distribution is
  * Copyright 2004 University of Washington.
@@ -21,6 +21,8 @@
 #include "ip_nt.c"
 
 
+#define TCPMAXSEND 32768
+
 /* Private functions */
 
 int tcp_socket_open (int family,void *adr,size_t adrlen,unsigned short port,
@@ -133,7 +135,7 @@ TCPSTREAM *tcp_open (char *host,char *service,unsigned long port)
       mm_log (tmp,TCPDEBUG);
     }
     (*bn) (BLOCK_DNSLOOKUP,NIL);/* look up name */
-    if (s = ip_nametoaddr (host,&adrlen,&family,&hostname,&next))
+    if (!(s = ip_nametoaddr (host,&adrlen,&family,&hostname,&next)))
       sprintf (tmp,"Host not found (#%d): %s",WSAGetLastError (),host);
     (*bn) (BLOCK_NONE,NIL);
     if (s) {			/* DNS resolution won? */
@@ -453,9 +455,11 @@ long tcp_sout (TCPSTREAM *stream,char *string,unsigned long size)
       return tcp_abort (&stream->tcpsi);
     default:
       if (stream->tcpsi == stream->tcpso)
-	while (((i = send (stream->tcpso,string,(int) size,0)) ==
-		SOCKET_ERROR) && (WSAGetLastError () == WSAEINTR));
-      else while (((i = write (stream->tcpso,string,size)) < 0) &&
+	while (((i = send (stream->tcpso,string,
+			   (int) min (size,TCPMAXSEND),0)) == SOCKET_ERROR) &&
+	       (WSAGetLastError () == WSAEINTR));
+      else while (((i = write (stream->tcpso,string,
+			       min (size,TCPMAXSEND))) < 0) &&
 		  (errno == EINTR));
       if (i == SOCKET_ERROR) return tcp_abort (&stream->tcpsi);
       size -= i;		/* count this size */
