@@ -1,5 +1,5 @@
 /*
- * Program:	Operating-system dependent routines -- OSF/1 version
+ * Program:	Operating-system dependent routines -- OSF/Digital UNIX/Tru64
  *
  * Author:	Mark Crispin
  *		Networks and Distributed Computing
@@ -10,10 +10,10 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	1 August 1988
- * Last Edited:	24 October 2000
+ * Last Edited:	15 June 2004
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 2000 University of Washington.
+ * Copyright 1988-2004 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
@@ -43,4 +43,4 @@ extern int errno;		/* just in case */
 #include "tcp_unix.c"
 #include "tz_bsd.c"
 #undef flock
-#include "flcksafe.c"
+#include "flocksim.c"
