@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	1 August 1988
- * Last Edited:	15 March 2004
+ * Last Edited:	26 May 2004
  * 
  * The IMAP toolkit provided in this Distribution is
  * Copyright 2004 University of Washington.
@@ -173,7 +173,7 @@ TCPSTREAM *tcp_open (char *host,char *service,unsigned long port)
     }
     (*bn) (BLOCK_DNSLOOKUP,NIL);/* quell alarms */
     data = (*bn) (BLOCK_SENSITIVE,NIL);
-    if (s = ip_nametoaddr (host,&adrlen,&family,&hostname,&next))
+    if (!(s = ip_nametoaddr (host,&adrlen,&family,&hostname,&next)))
       sprintf (tmp,"No such host as %.80s",host);
     (*bn) (BLOCK_NONSENSITIVE,data);
     (*bn) (BLOCK_NONE,NIL);
