@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: attach.c,v 4.69 2004/05/07 21:35:09 jpf Exp $";
+static char rcsid[] = "$Id: attach.c,v 4.71 2004/06/15 22:22:35 hubert Exp $";
 #endif
 /*
  * Program:	Routines to support attachments in the Pine composer 
@@ -57,12 +57,16 @@ static char rcsid[] = "$Id: attach.c,v 4.69 2004/05/07 21:35:09 jpf Exp $";
  * AskAttach - ask for attachment fields and build resulting structure
  *             return pointer to that struct if OK, NULL otherwise
  */
-AskAttach(fn, sz, cmnt)
-char *fn, *sz, *cmnt;
+AskAttach(cmnt, lm)
+char    *cmnt;
+LMLIST **lm;
 {
     int	    i, status, fbrv, upload = 0;
+    int     fb_flags = FB_READ | FB_ATTACH;
     off_t   attsz = 0;
     char    bfn[NLINE];
+    char    fn[NLINE], sz[32];
+    LMLIST *new;
 
     i = 2;
     fn[0] = '\0';
@@ -195,19 +199,45 @@ char *fn, *sz, *cmnt;
 		     ? opertree 
 		     : (browse_dir[0] ? browse_dir
 			: gethomedir(NULL)));
-	    if((fbrv = FileBrowse(fn, NLINE, bfn, NLINE, sz, FB_READ | FB_ATTACH)) == 1){
-	      if ((strlen(fn)+strlen(S_FILESEP)+strlen(bfn)) < NLINE){
+	    if((fbrv = FileBrowse(fn, NLINE, bfn, NLINE, sz,
+				  upload ? fb_flags
+				         : fb_flags|FB_LMODEPOS,
+				  upload ? NULL
+					 : lm)) == 1){
+	      if (upload && (strlen(fn)+strlen(S_FILESEP)+strlen(bfn)) < NLINE){
+
+		if((new=(LMLIST *)malloc(sizeof(*new))) == NULL
+		   || (new->fname=malloc(strlen(bfn)+1)) == NULL
+		   || (new->dir=malloc(strlen(fn)+1)) == NULL){
+		    emlwrite("\007Can't malloc space for filename", NULL);
+		    return(-1);
+		}
+
+		strcpy(new->fname, bfn);
+		strcpy(new->dir, fn);
+		strncpy(new->size, sz, sizeof(new->size)-1);
+		new->size[sizeof(new->size)-1] = '\0';
+		new->next = NULL;
+		*lm = new;
+
 		strcat(fn, S_FILESEP);
 		strcat(fn, bfn);
-		if(upload && !AttachUpload(fn, sz)){              
+		if(!AttachUpload(fn, sz)){              
 		  i = 2;                  /* keep prompting for file */
 		  sleep(3);                       /* problem, show error! */
 		}
 		else{
-		  (void) QuoteAttach(fn);
 		  i--;                    /* go prompt for comment */
 		}
 	      }
+	      else if(!upload){
+		  if(lm && *lm && !(*lm)->next)	/* get comment */
+		    i--;
+		  else{			/* no comments if multiple files */
+		      update();
+		      return(1);
+		  }
+	      }
 	      else{                           /* trouble */
 		*fn = '\0';
 		AttachCancel(fn);
@@ -294,8 +324,17 @@ char *fn, *sz, *cmnt;
 			    return(0);
 			}
 
-			QuoteAttach(fn);
-			strcpy(sz, prettysz(attsz));
+			if((new=(LMLIST *)malloc(sizeof(*new))) == NULL
+			   || (new->fname=malloc(strlen(fn)+1)) == NULL){
+			    emlwrite("\007Can't malloc space for filename", NULL);
+			    return(-1);
+			}
+
+			new->dir = NULL;
+			strcpy(new->fname, fn);
+			strcpy(new->size, prettysz(attsz));
+			new->next = NULL;
+			*lm = new;
 		    }
 		    else
 		      return(AttachCancel((upload && i == 1) ? fn : NULL));
@@ -1211,6 +1250,7 @@ char *prettysz(l)
     long sz, left, right;
 
     sz = (long) l;
+    b[0] = '\0';
 
     if(sz < 1000L){
 	sprintf(b, "%ld B", sz);				/* xxx B */
