@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: display.c,v 4.107 2002/12/04 01:15:07 hubert Exp $";
+static char rcsid[] = "$Id: display.c,v 4.109 2004/05/07 21:38:21 jpf Exp $";
 #endif
 /*
  * Program:	Display functions
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: display.c,v 4.107 2002/12/04 01:15:07 hubert Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-2001 by the University of Washington.
+ * 1989-2004 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -758,8 +758,10 @@ out:
 
 	    if(lastflag&CFFILL){
 		menu_pico[UNCUT_KEY].label = "UnJustify";
-		emlwrite("Can now UnJustify!", NULL);
-		mpresf = FARAWAY;	/* remove this after next keystroke! */
+		if(!(lastflag&CFFLBF)){
+		    emlwrite("Can now UnJustify!", NULL);
+		    mpresf = FARAWAY;	/* remove this after next keystroke! */
+		}
 	    }
 	    else
 	      menu_pico[UNCUT_KEY].label = "UnCut Text";
@@ -768,6 +770,10 @@ out:
 	    sgarbk = FALSE;
         }
     }
+    if(lastflag&CFFLBF){
+	emlwrite("Can now UnJustify!", NULL);
+	mpresf = FARAWAY;  /* remove this after next keystroke! */
+    }
 
     /* Finally, update the hardware cursor and flush out buffers. */
 
