@@ -1,5 +1,5 @@
 /*
- * $Id: pico.h,v 4.133 2004/05/07 19:35:04 jpf Exp $
+ * $Id: pico.h,v 4.136 2004/12/01 22:34:14 jpf Exp $
  *
  * Program:	pico.h - definitions for Pine's composer library
  *
@@ -412,7 +412,7 @@ extern	void	kbdestroy PROTO((KBESC_T *));
  */
 #ifdef	maindef
 PICO	*Pmaster = NULL;		/* composer specific stuff */
-char	*version = "4.7";		/* PICO version number */
+char	*version = "4.9";		/* PICO version number */
 
 #else
 extern	PICO *Pmaster;			/* composer specific stuff */
@@ -427,6 +427,8 @@ extern	char *version;			/* pico version! */
 #define FB_READ		0x0001		/* Looking for a file to read.  */
 #define FB_SAVE		0x0002		/* Looking for a file to save.  */
 #define	FB_ATTACH	0x0004		/* Looking for a file to attach */
+#define	FB_LMODEPOS	0x0008		/* ListMode is a possibility    */
+#define	FB_LMODE	0x0010		/* Using ListMode now           */
 
 
 /*
