@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: imap.c,v 4.267 2004/05/07 22:31:13 jpf Exp $";
+static char rcsid[] = "$Id: imap.c,v 4.269 2004/10/14 01:41:02 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -1585,6 +1585,7 @@ pine_mail_status_full(stream, mailbox, flags, uidvalidity, uidnext)
 		 * if we add messages, we could get it wrong. As far as I
 		 * can tell, Pine doesn't ever even use status.unseen, so it
 		 * is currently academic anyway.)  Hubert 2003-03-05
+		 * (Does now 2004-06-02 in next_folder.)
 		 *
 		 * However, we don't want to count all messages as recent if
 		 * the source mailbox is NNTP or NEWS, because we won't be
@@ -1700,15 +1701,22 @@ pine_mail_status_full(stream, mailbox, flags, uidvalidity, uidnext)
 		 * We just use this to find the answer.
 		 * We're asking for trouble if we do a STATUS on a
 		 * selected mailbox. I don't believe this happens in pine.
+		 * It does now (2004-06-02) in next_folder if the
+		 * F_TAB_USES_UNSEEN option is set and the folder was
+		 * already opened.
 		 */
 		stream = sp_stream_get(mailbox, SP_MATCH);
 		if(stream){ 
 		    memset(&status, 0, sizeof(status));
-		    if(flags & SA_MESSAGES)
-		      status.messages = stream->nmsgs;
+		    if(flags & SA_MESSAGES){
+			status.flags |= SA_MESSAGES;
+			status.messages = stream->nmsgs;
+		    }
 
-		    if(flags & SA_RECENT)
-		      status.recent = stream->recent;
+		    if(flags & SA_RECENT){
+			status.flags |= SA_RECENT;
+			status.recent = stream->recent;
+		    }
 
 		    if(flags & SA_UNSEEN){
 		        long i;
@@ -1720,17 +1728,22 @@ pine_mail_status_full(stream, mailbox, flags, uidvalidity, uidnext)
 		      
 			pine_mail_search_full(stream, NULL, srchpgm,
 					      SE_NOPREFETCH | SE_FREE);
+			status.flags |= SA_UNSEEN;
 			status.unseen = 0L;
 			for(i = 1L; i <= stream->nmsgs; i++)
 			  if((mc = mail_elt(stream, i)) && mc->searched)
-			    srchpgm->unseen++;
+			    status.unseen++;
 		    }
 
-		    if(flags & SA_UIDVALIDITY)
-		      status.uidvalidity = stream->uid_validity;
+		    if(flags & SA_UIDVALIDITY){
+			status.flags |= SA_UIDVALIDITY;
+			status.uidvalidity = stream->uid_validity;
+		    }
 
-		    if(flags & SA_UIDNEXT)
-		      status.uidnext = stream->uid_last + 1L;
+		    if(flags & SA_UIDNEXT){
+			status.flags |= SA_UIDNEXT;
+			status.uidnext = stream->uid_last + 1L;
+		    }
 
 		    mm_status(NULL, mailbox, &status);
 		    return T;  /* that's what c-client returns when success */
@@ -3026,6 +3039,8 @@ imap_flush_passwd_cache()
     MMLOGIN_S *l;
 
     memset((void *)private_store, 0, sizeof(private_store));
+    if(panicking)
+      return;
 
     while(l = mm_login_list){
 	mm_login_list = mm_login_list->next;
