@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: mailcap.c,v 1.86 2004/05/07 22:51:09 jpf Exp $";
+static char rcsid[] = "$Id: mailcap.c,v 1.87 2004/11/03 20:11:00 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -167,7 +167,8 @@ int	  mc_comment PROTO((char **));
 int	  mc_token PROTO((char **, char **));
 void	  mc_build_entry PROTO((char **));
 char     *mc_bld_test_cmd PROTO((char *, int, char *, PARAMETER *));
-char     *mc_cmd_bldr PROTO((char *, int, char *, PARAMETER *, char *));
+char     *mc_cmd_bldr PROTO((char *, int, char *, PARAMETER *, char *,
+			     char **));
 int       mc_ctype_match PROTO((int, char *, char *));
 int	  mc_sane_command PROTO((char *));
 MailcapEntry *mc_get_command PROTO((int, char *, PARAMETER *, int, int *));
@@ -398,6 +399,11 @@ int
 mc_comment(line)
     char **line;
 {
+    if(**line == '\n'){		/* blank line is a comment, too */
+	(*line)++;
+	return(1);
+    }
+
     if(**line == '#'){
 	while(**line)			/* !EOF */
 	  if(*++(*line) == '\n'){	/* EOL? */
@@ -443,6 +449,19 @@ mc_token(token, line)
 	    *start = '\0';		/* if we have a token, tie it off  */
 
 	  (*line)++;			/* and get ready to parse next one */
+
+	  if(rv == 1){			/* ignore trailing semicolon */
+	      while(**line){
+		  if(**line == '\n')
+		    rv = 0;
+		  
+		  if(isspace((unsigned char) **line))
+		    (*line)++;
+		  else
+		    break;
+	      }
+	  }
+
 	case '\0' :			/* EOF */
 	  return(rv);
 
@@ -817,7 +836,7 @@ mailcap_build_command(type, subtype, params, tmp_file, needsterm, chk_extension)
     int        chk_extension;
 {
     MailcapEntry *mc;
-    char         *command;
+    char         *command, *err = NULL;
     MCAP_CMD_S   *mc_cmd = NULL;
     int           sp_handling = 0;
 
@@ -837,8 +856,8 @@ mailcap_build_command(type, subtype, params, tmp_file, needsterm, chk_extension)
 
     if(sp_handling)
       command = cpystr(mc->command);
-    else if(!(command = mc_cmd_bldr(mc->command, type, subtype, params, tmp_file)))
-      command = cpystr("");
+    else if(!(command = mc_cmd_bldr(mc->command, type, subtype, params, tmp_file, &err)) && err && *err)
+      q_status_message(SM_ORDER, 5, 5, err);
 
     dprint(5, (debugfile, "built command: %s\n", command ? command : "?"));
 
@@ -865,7 +884,7 @@ mc_bld_test_cmd(controlstring, type, subtype, params)
     char      *subtype;
     PARAMETER *params;
 {
-    return(mc_cmd_bldr(controlstring, type, subtype, params, NULL));
+    return(mc_cmd_bldr(controlstring, type, subtype, params, NULL, NULL));
 }
 
 
@@ -879,12 +898,13 @@ mc_bld_test_cmd(controlstring, type, subtype, params)
  *    The return value is an alloc'd copy of the command to be executed.
  */
 char *
-mc_cmd_bldr(controlstring, type, subtype, parameter, tmp_file)
+mc_cmd_bldr(controlstring, type, subtype, parameter, tmp_file, err)
     char *controlstring;
     int   type;
     char *subtype;
     PARAMETER *parameter;
     char *tmp_file;
+    char **err;
 {
     char        *from, *to, *s, *parm;
     int	         prefixed = 0, used_tmp_file = 0;
@@ -924,7 +944,19 @@ mc_cmd_bldr(controlstring, type, subtype, parameter, tmp_file)
 		    dprint(2,
 			   (debugfile, "mc_cmd_bldr: param subs %s\n",
 			   from ? from : "?"));
-		   return(NULL);
+		    if(err){
+			if(s = strindex(from, '}'))
+			  *++s = '\0';
+
+			sprintf(tmp_20k_buf,
+			    "Mailcap: see hidden feature %.200s (%%%.200s)",
+			    feature_list_name(F_DO_MAILCAP_PARAM_SUBST), from);
+			*err = tmp_20k_buf;
+			if(s)
+			  *s = '}';
+		    }
+
+		    return(NULL);
 		}
 
 		s = strindex(from, '}');
