@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: screen.c,v 4.148 2004/04/07 17:35:24 hubert Exp $";
+static char rcsid[] = "$Id: screen.c,v 4.149 2004/12/01 18:55:52 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -139,11 +139,13 @@ int		 row,
     off    	  = km->which * 12;
     max_column    = ps_global->ttyo->screen_cols;
 
-    if(ps_global->ttyo->screen_rows < 4 || max_column <= 0){
+    if((ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global)) < 0
+       || max_column <= 0){
 	keymenu_is_dirty = 1;
 	return;
     }
 
+
     real_row = row > 0 ? row : ps_global->ttyo->screen_rows + row;
 
     if(pico_usingcolor()){
@@ -1223,6 +1225,12 @@ output_titlebar(tc)
 {
     COLOR_PAIR *lastc = NULL, *newcolor;
 
+    if(ps_global->ttyo
+       && (ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global)) < 1){
+	titlebar_is_dirty = 1;
+	return;
+    }
+
     newcolor = tc ? &tc->color : NULL;
 
     if(newcolor)
@@ -1575,6 +1583,12 @@ update_titlebar_message()
     long oldnum;
     PINETHRD_S *thrd = NULL;
 
+    if(ps_global->ttyo
+       && (ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global)) < 1){
+	titlebar_is_dirty = 1;
+	return;
+    }
+
     if(as.style == ThrdIndex){
 	unsigned long rawno;
 
@@ -1669,6 +1683,12 @@ update_titlebar_status()
     MESSAGECACHE *mc;
     COLOR_PAIR *lastc = NULL, *titlecolor;
     
+    if(ps_global->ttyo
+       && (ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global)) < 1){
+	titlebar_is_dirty = 1;
+	return(0);
+    }
+
     if(!as.stream || as.current_msg <= 0L || as.del_column < 0)
       return(1);
 
@@ -1738,6 +1758,12 @@ update_titlebar_percent(new_line_number)
     if(as.percent_column < 0 || new_line_number == as.current_line)
       return;
 
+    if(ps_global->ttyo
+       && (ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global)) < 1){
+	titlebar_is_dirty = 1;
+	return;
+    }
+
     as.current_line = new_line_number;
 
     titlecolor = &as.titlecontainer.color;
@@ -1774,6 +1800,12 @@ update_titlebar_lpercent(new_line_number)
     if(as.page_column < 0 || new_line_number == as.current_line)
       return;
 
+    if(ps_global->ttyo
+       && (ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global)) < 1){
+	titlebar_is_dirty = 1;
+	return;
+    }
+
     as.current_line = new_line_number;
 
 
