@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: signals.c,v 4.159 2004/05/07 22:56:45 jpf Exp $";
+static char rcsid[] = "$Id: signals.c,v 4.162 2004/11/04 22:31:46 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -76,6 +76,7 @@ static	SigType	alarm_signal SIG_PROTO((int));
 static	SigType	intr_signal SIG_PROTO((int));
 
 static  int now_handling_alarms;
+static  int cleanup_called_from_sig_handler;
 
 
 
@@ -227,6 +228,7 @@ auger_in_signal SIG_PROTO ((int sig))
 {
     char buf[100], *s;
 
+    panicking = 1;
     end_signals(1);			/* don't catch any more signals */
     imap_flush_passwd_cache();
 
@@ -236,8 +238,6 @@ auger_in_signal SIG_PROTO ((int sig))
     s = "?";
 #endif
 
-    dprint(5, (debugfile, "auger_in_signal(sig=%s)\n", s));
-
     sprintf(buf, "Received abort signal(sig=%.9s)", s);
     panic(buf);				/* clean up and get out */
 
@@ -289,21 +289,16 @@ Not much to do. Rely on periodic mail file check pointing.
 SigType
 hup_signal()
 {
-#if	!defined(DOS) || defined(_WINDOWS)
     end_signals(1);			/* don't catch any more signals */
     dprint(1, (debugfile, "\n\n** Received SIGHUP **\n\n\n\n"));
+    cleanup_called_from_sig_handler = 1;
     fast_clean_up();
 #if	defined(DEBUG)
     if(debugfile)
       fclose(debugfile);
 #endif 
-#ifdef	_WINDOWS
+
     _exit(0);				/* cleaning up can crash */
-#else
-    printf("\n\nPine finished. Received hang up signal\n\n");
-#endif
-#endif	/* !DOS */
-    exit(0);
 }
 
 
@@ -349,6 +344,7 @@ term_signal()
 #if !defined(DOS) && !defined(OS2)
     end_signals(1);			/* don't catch any more signals */
     dprint(1, (debugfile, "\n\n** Received SIGTERM **\n\n\n\n"));
+    cleanup_called_from_sig_handler = 1;
     fast_clean_up();
 #if	defined(DEBUG) && (!defined(DOS) || defined(_WINDOWS))
     if(debugfile)
@@ -393,7 +389,8 @@ fast_clean_up()
      */
     if(ps_global->prc){
 	if(ps_global->prc->outstanding_pinerc_changes)
-	  write_pinerc(ps_global, Main);
+	  write_pinerc(ps_global, Main,
+		       cleanup_called_from_sig_handler ? WRP_NOUSER : WRP_NONE);
 
 	if(ps_global->prc->rd)
 	  rd_close_remdata(&ps_global->prc->rd);
@@ -404,7 +401,8 @@ fast_clean_up()
     /* as does this */
     if(ps_global->post_prc){
 	if(ps_global->post_prc->outstanding_pinerc_changes)
-	  write_pinerc(ps_global, Post);
+	  write_pinerc(ps_global, Post,
+		       cleanup_called_from_sig_handler ? WRP_NOUSER : WRP_NONE);
 
 	if(ps_global->post_prc->rd)
 	  rd_close_remdata(&ps_global->post_prc->rd);
