@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: status.c,v 4.134 2004/05/07 22:56:45 jpf Exp $";
+static char rcsid[] = "$Id: status.c,v 4.138 2005/01/13 22:12:06 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -142,6 +142,20 @@ q_status_message(flags, min_time, max_time, message)
     char  *clean_msg;
     size_t mlen;
 
+    /*
+     * By convention, we have min_time equal to zero in messages which we
+     * think are not as important, so-called comfort messages. We have
+     * min_time >= 3 for messages which we think the user should see for
+     * sure. Some users don't like to wait so we've provided a way for them
+     * to live on the edge.
+     *    status_msg_delay == -1  => min time == min(0, min_time)
+     *    status_msg_delay == -2  => min time == min(1, min_time)
+     *    status_msg_delay == -3  => min time == min(2, min_time)
+     *    ...
+     */
+    if(ps_global->status_msg_delay < 0)
+      min_time = min(-1 - ps_global->status_msg_delay, min_time);
+
     /*
      * The 41 is room for 40 escaped control characters plus a
      * terminating null.
@@ -526,7 +540,7 @@ display_message(command)
     int command;
 {
     if(ps_global == NULL || ps_global->ttyo == NULL
-       || ps_global->ttyo->screen_rows <= 1 || ps_global->in_init_seq)
+       || ps_global->ttyo->screen_rows < 1 || ps_global->in_init_seq)
       return(0);
 
     /*---- Deal with any previously displayed messages ----*/
@@ -550,7 +564,7 @@ display_message(command)
 			    : message_queue->min_display_time);
             dprint(9, (debugfile,
 		       "STATUS: diff:%d, displayed: %ld, now: %ld\n",
-		       diff, displayed_time, now));
+		       diff, (long) displayed_time, (long) now));
             if(diff > 0)
 	      rv = diff;			/* check again next time  */
 	    else if(LAST_MESSAGE(message_queue)
@@ -908,7 +922,7 @@ output_message(mq_entry)
 
     if(!(mq_entry->flags & SM_MODAL)){
 	rv = status_message_write(mq_entry->text, 0);
-    	if(ps_global->status_msg_delay){
+    	if(ps_global->status_msg_delay > 0){
 	    MoveCursor(ps_global->ttyo->screen_rows-FOOTER_ROWS(ps_global), 0);
 	    fflush(stdout);
 	    sleep(ps_global->status_msg_delay);
@@ -1245,6 +1259,11 @@ Args --  prompt:    The prompt for the question/selection
 				   changes so return on_ctrl_C if an
 				   unsolicited expunge happens while we're
 				   viewing a message.
+		RB_RET_HELP    - Instead of the regular internal handling
+				 way of handling help_text, this just causes
+				 radio_buttons to return 3 when help is
+				 asked for, so that the caller handles it
+				 instead.
 	
 	 Note: If there are enough keys in the esc_list to need a second
 	       screen, and there is no help, then the 13th key will be
@@ -1284,14 +1303,26 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 
 #ifdef _WINDOWS
     if (mswin_usedialog ()) {
-	MDlgButton		button_list[24];
+	MDlgButton		button_list[25];
 	int			b;
 	int			i;
 	int			ret;
 	char			**help;
 
-	memset (&button_list, 0, sizeof (MDlgButton) * 24);
+	memset (&button_list, 0, sizeof (MDlgButton) * 25);
 	b = 0;
+
+	if(flags & RB_RET_HELP){
+	    if(help_text != NO_HELP)
+		panic("RET_HELP and help in radio_buttons!");
+
+	    button_list[b].ch = '?';
+	    button_list[b].rval = 3;
+	    button_list[b].name = "?";
+	    button_list[b].label = "Help";
+	    ++b;
+	}
+
 	for (i = 0; esc_list && esc_list[i].ch != -1 && i < 23; ++i) {
 	  if(esc_list[i].ch != -2){
 	    button_list[b].ch = esc_list[i].ch;
@@ -1347,7 +1378,11 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
     start = 0;
     clrbitmap(bitmap);
     memset(fkey_table, NO_OP_COMMAND, 12 * sizeof(int));
-    if(help_text != NO_HELP){		/* if shown, always at position 0 */
+    if(flags & RB_RET_HELP && help_text != NO_HELP)
+      panic("RET_HELP and help in radio_buttons!");
+
+    /* if shown, always at position 0 */
+    if(help_text != NO_HELP || flags & RB_RET_HELP){
 	rb_keymenu.keys[0].name  = "?";
 	rb_keymenu.keys[0].label = "Help";
 	setbitn(0, bitmap);
@@ -1562,7 +1597,11 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 		break;
 	    }
 
-	    if(help_text != NO_HELP && FOOTER_ROWS(ps_global) > 1){
+	    if(flags & RB_RET_HELP){
+		ch = 3;
+		goto out_of_loop;
+	    }
+	    else if(help_text != NO_HELP && FOOTER_ROWS(ps_global) > 1){
 		mark_keymenu_dirty();
 		if(lastc)
 		  (void)pico_set_colorp(lastc, PSC_NONE);
